@@ -25,18 +25,36 @@
 #import "dispatch/dispatch.h"
 #import "DdbWidgetManager.h"
 #import "DdbPlaylistViewController.h"
+#import "ReplayGainScannerController.h"
 #import "DdbShared.h"
+#import "MediaKeyController.h"
+#import "LogWindowController.h"
+#import "deadbeef-Swift.h"
 #include "conf.h"
 #include "streamer.h"
 #include "junklib.h"
-
+#include "../../common.h"
 #include "../../deadbeef.h"
+#include "../../plugins/artwork/artwork.h"
 #include <sys/time.h>
 
 extern DB_functions_t *deadbeef;
 
 extern BOOL g_CanQuit;
-@implementation AppDelegate
+@implementation AppDelegate {
+    PreferencesWindowController *_prefWindow;
+    SearchWindowController *_searchWindow;
+    LogWindowController *_logWindow;
+    HelpWindowController *_helpWindow;
+
+    NSMenuItem *_dockMenuNPHeading;
+    NSMenuItem *_dockMenuNPTitle;
+    NSMenuItem *_dockMenuNPArtistAlbum;
+    NSMenuItem *_dockMenuNPSeparator;
+
+    char *_titleScript;
+    char *_artistAlbumScript;
+}
 
 DB_playItem_t *prev = NULL;
 int prevIdx = -1;
@@ -46,8 +64,33 @@ NSImage *bufferingImg;
 AppDelegate *g_appDelegate;
 NSInteger firstSelected = -1;
 
-- (NSWindow *)mainWindow {
-    return [_mainWindow window];
+static void
+_cocoaui_logger_callback (DB_plugin_t *plugin, uint32 layers, const char *text, void *ctx) {
+    [g_appDelegate appendLoggerText:text forPlugin:plugin onLayers:layers];
+}
+
+- (void)appendLoggerText:(const char *)text forPlugin:(DB_plugin_t *)plugin onLayers:(uint32_t)layers {
+    NSString *str = [NSString stringWithUTF8String:text];
+    if (!str) {
+        return; // may happen in case of invalid UTF8 and such
+    }
+
+    dispatch_async(dispatch_get_main_queue(), ^{
+        [_logWindow appendText:str];
+
+        if (layers == DDB_LOG_LAYER_DEFAULT) {
+            if (![[_logWindow window] isVisible]) {
+                [_logWindow showWindow:self];
+            }
+        }
+    });
+}
+
+- (void)dealloc {
+    deadbeef->log_viewer_unregister (_cocoaui_logger_callback, NULL);
+#if !DISABLE_MM_KEY_GRABBER
+    ungrabMediaKeys ();
+#endif
 }
 
 - (void)volumeChanged {
@@ -85,13 +128,10 @@ NSInteger firstSelected = -1;
     [_cursorFollowsPlayback setState:deadbeef->conf_get_int ("playlist.scroll.cursorfollowplayback", 1)?NSOnState:NSOffState];
     
     [_stopAfterCurrent setState:deadbeef->conf_get_int ("playlist.stop_after_current", 0)?NSOnState:NSOffState];
-    [_stopAfterCurrentAlbum setState:deadbeef->conf_get_int ("playlist.stop_after_current_album", 0)?NSOnState:NSOffState];
+    [_stopAfterCurrentAlbum setState:deadbeef->conf_get_int ("playlist.stop_after_album", 0)?NSOnState:NSOffState];
 
     [_descendingSortMode setState:deadbeef->conf_get_int ("cocoaui.sort_desc", 0) ? NSOnState : NSOffState];
 
-    conf_save ();
-    streamer_configchanged ();
-    junk_configchanged ();
     [self volumeChanged];
 
     [_mainWindow updateTitleBarConfig];
@@ -103,7 +143,7 @@ static int fileadd_cancelled = 0;
 static void fileadd_begin (ddb_fileadd_data_t *data, void *user_data) {
     fileadd_cancelled = 0;
     dispatch_async(dispatch_get_main_queue(), ^{
-        [NSApp beginSheet:g_appDelegate.addFilesWindow modalForWindow:[g_appDelegate mainWindow] modalDelegate:nil didEndSelector:nil contextInfo:nil];
+        [NSApp beginSheet:g_appDelegate.addFilesWindow modalForWindow:[[g_appDelegate mainWindow] window] modalDelegate:nil didEndSelector:nil contextInfo:nil];
     });
 }
 
@@ -111,16 +151,25 @@ static void fileadd_end (ddb_fileadd_data_t *data, void *user_data) {
     dispatch_async(dispatch_get_main_queue(), ^{
         [g_appDelegate.addFilesWindow orderOut:g_appDelegate];
         [NSApp endSheet:g_appDelegate.addFilesWindow];
-        [g_appDelegate.mainWindow makeKeyAndOrderFront:g_appDelegate];
+        [[[g_appDelegate mainWindow] window] makeKeyAndOrderFront:g_appDelegate];
     });
 }
 
+static BOOL _settingLabel = NO;
+
 static int file_added (ddb_fileadd_data_t *data, void *user_data) {
     const char *uri = deadbeef->pl_find_meta (data->track, ":URI");
-    NSString *s = [NSString stringWithUTF8String:uri];
-    dispatch_sync(dispatch_get_main_queue(), ^{
-        [g_appDelegate.addFilesLabel setStringValue:s];
-    });
+    if (!_settingLabel) {
+        // HACK: we want to set the label asynchronously, to minimize delays,
+        // but we also want to avoid sending multiple labels, becuase that's meaningless.
+        // So we use a basic flag, to see if the label is being set already.
+        NSString *s = [NSString stringWithUTF8String:uri];
+        _settingLabel = YES;
+        dispatch_async(dispatch_get_main_queue(), ^{
+            [g_appDelegate.addFilesLabel setStringValue:s];
+            _settingLabel = NO;
+        });
+    }
     return fileadd_cancelled ? -1 : 0;
 }
 
@@ -131,24 +180,16 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
 - (void)awakeFromNib {
     [self initMainWindow];
     [self initSearchWindow];
+    [self initLogWindow];
 }
 
 - (void)initMainWindow {
     _mainWindow = [[MainWindowController alloc] initWithWindowNibName:@"MainWindow"];
     [_mainWindow setShouldCascadeWindows:NO];
     [[_mainWindow window] setReleasedWhenClosed:NO];
-    [[_mainWindow window]  setExcludedFromWindowsMenu:YES];
+    [[_mainWindow window] setExcludedFromWindowsMenu:YES];
     [[_mainWindow window] setIsVisible:YES];
-
-/*
-    DdbPlaylistViewController *vc = [[DdbPlaylistViewController alloc] init];
-    NSView *view = [vc view];
-
-    [view setFrame:NSMakeRect(0, [_statusBar frame].size.height+2, [[_window contentView] frame].size.width, [_tabStrip frame].origin.y - [_statusBar frame].size.height - 1)];
-    [view setAutoresizingMask:NSViewWidthSizable|NSViewHeightSizable];
-
-    [[_window contentView] addSubview:view];
-    */
+    [_mainWindowToggleMenuItem bind:@"state" toObject:[_mainWindow window] withKeyPath:@"visible" options:nil];
 }
 
 - (void)initSearchWindow {
@@ -156,20 +197,49 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
     [_searchWindow setShouldCascadeWindows:NO];
 }
 
+- (void)initLogWindow {
+    _logWindow = [[LogWindowController alloc] initWithWindowNibName:@"Log"];
+    [_logWindowToggleMenuItem bind:@"state" toObject:[_logWindow window] withKeyPath:@"visible" options:nil];
+    [[_logWindow window] setExcludedFromWindowsMenu:YES];
+}
+
 - (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender {
     if (g_CanQuit) {
         return NSTerminateNow;
     }
 
-    [_mainWindow close];
-    [_searchWindow close];
+    if (deadbeef->have_background_jobs ()) {
+        NSAlert *alert = [[NSAlert alloc] init];
+        [alert setMessageText:@"Do you really want to quit now?"];
+        [alert setInformativeText:@"DeaDBeeF is currently running background tasks. If you quit now, the tasks will be cancelled or interrupted. This may result in data loss."];
+        [alert addButtonWithTitle:@"No"];
+        [alert addButtonWithTitle:@"Yes"];
+        NSModalResponse res = [alert runModal];
+        if (res == NSAlertFirstButtonReturn) {
+            return NSTerminateCancel;
+        }
+    }
 
     deadbeef->sendmessage(DB_EV_TERMINATE, 0, 0, 0);
     return NSTerminateLater;
 }
 
+extern void
+main_cleanup_and_quit (void);
+
+- (void)applicationWillTerminate:(NSNotification *)notification {
+    [ConverterWindowController cleanup];
+    [ReplayGainScannerController cleanup];
+    [_mainWindow cleanup];
+    [_searchWindow cleanup];
+    main_cleanup_and_quit();
+}
+
 - (void)applicationDidFinishLaunching:(NSNotification *)aNotification
 {
+    // high sierra would terminate the app on SIGPIPE by default, which breaks converter error handling
+    signal(SIGPIPE, SIG_IGN);
+
     playImg = [NSImage imageNamed:@"btnplayTemplate.pdf"];
     pauseImg = [NSImage imageNamed:@"btnpauseTemplate.pdf"];
     bufferingImg = [NSImage imageNamed:@"bufferingTemplate.pdf"];
@@ -182,10 +252,16 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
 
     g_appDelegate = self;
 
+#if !DISABLE_MM_KEY_GRABBER
+    grabMediaKeys ();
+#endif
+
     [self updateDockNowPlaying];
     [[NSApp dockTile] setContentView: _dockTileView];
 //    [[NSApp dockTile] setBadgeLabel:@"Hello"];
     [[NSApp dockTile] display];
+
+    deadbeef->log_viewer_register (_cocoaui_logger_callback, NULL);
 }
 
 - (BOOL)applicationShouldHandleReopen:(NSApplication *)sender hasVisibleWindows:(BOOL)flag{
@@ -193,10 +269,41 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
     return YES;
 }
 
+- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename {
+    dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
+    dispatch_async(aQueue, ^{
+        char str[100];
+        add_paths([filename UTF8String], (int)[filename length], 0, str, 100);
+    });
+    return YES; // assume that everything went ok
+}
+
+
+- (void)application:(NSApplication *)sender openFiles:(NSArray *)filenames {
+    dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
+    dispatch_async(aQueue, ^{
+        char str[100];
+        // building single paths string for the deadbeef function, paths must be separated by '\0'
+        NSString *paths =[filenames componentsJoinedByString:@"\0"];
+        add_paths([paths UTF8String], (int)[paths length], 0, str, 100);
+    });
+}
+
+
 - (IBAction)showMainWinAction:(id)sender {
-    NSInteger st = [sender state];
-    [[_mainWindow window] setIsVisible:st!=NSOnState];
-    [sender setState:st==NSOnState?NSOffState:NSOnState];
+    BOOL vis = ![[_mainWindow window] isVisible];
+    [[_mainWindow window] setIsVisible:vis];
+    if (vis) {
+        [[_mainWindow window] makeKeyWindow];
+    }
+}
+
+- (IBAction)showLogWindowAction:(id)sender {
+    BOOL vis = ![[_logWindow window] isVisible];
+    [[_logWindow window] setIsVisible:vis];
+    if (vis) {
+        [[_logWindow window] makeKeyWindow];
+    }
 }
 
 // playlist delegate
@@ -249,30 +356,31 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
     {
         NSArray* files = [openDlg URLs];
         ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-        if (clear) {
-            deadbeef->plt_clear(plt);
-            deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
-        }
-        if (plt) {
-            if (!deadbeef->plt_add_files_begin (plt, 0)) {
-                dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
-                dispatch_async(aQueue, ^{
-                    for( int i = 0; i < [files count]; i++ )
-                    {
-                        NSString* fileName = [[files objectAtIndex:i] path];
-                        if (fileName) {
-                            deadbeef->plt_add_file2 (0, plt, [fileName UTF8String], NULL, NULL);
-                        }
-                    }
-                    deadbeef->plt_add_files_end (plt, 0);
-                    deadbeef->plt_unref (plt);
-                    deadbeef->pl_save_current();
-                    if (play) {
-                        deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
-                        deadbeef->sendmessage (DB_EV_PLAY_NUM, 0, 0, 0);
-                    }
-                });
+        if (!deadbeef->plt_add_files_begin (plt, 0)) {
+            if (clear) {
+                deadbeef->plt_clear(plt);
+                deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
             }
+            dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
+            dispatch_async(aQueue, ^{
+                for( int i = 0; i < [files count]; i++ )
+                {
+                    NSString* fileName = [[files objectAtIndex:i] path];
+                    if (fileName) {
+                        deadbeef->plt_add_file2 (0, plt, [fileName UTF8String], NULL, NULL);
+                    }
+                }
+                deadbeef->plt_add_files_end (plt, 0);
+                deadbeef->plt_unref (plt);
+                deadbeef->pl_save_current();
+                if (play) {
+                    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
+                    deadbeef->sendmessage (DB_EV_PLAY_NUM, 0, 0, 0);
+                }
+            });
+        }
+        else {
+            deadbeef->plt_unref (plt);
         }
     }
 }
@@ -294,23 +402,24 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
     {
         NSArray* files = [openDlg URLs];
         ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-        if (plt) {
-            if (!deadbeef->plt_add_files_begin (plt, 0)) {
-                dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
-                dispatch_async(aQueue, ^{
-                    for( int i = 0; i < [files count]; i++ )
-                    {
-                        NSString *fileName = [[files objectAtIndex:i] path];
-                        if (fileName) {
-                            deadbeef->plt_add_dir2 (0, plt, [fileName UTF8String], NULL, NULL);
-                        }
+        if (!deadbeef->plt_add_files_begin (plt, 0)) {
+            dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
+            dispatch_async(aQueue, ^{
+                for( int i = 0; i < [files count]; i++ )
+                {
+                    NSString *fileName = [[files objectAtIndex:i] path];
+                    if (fileName) {
+                        deadbeef->plt_add_dir2 (0, plt, [fileName UTF8String], NULL, NULL);
                     }
-                    deadbeef->plt_add_files_end (plt, 0);
-                    deadbeef->plt_unref (plt);
-                    deadbeef->pl_save_current();
-                    deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
-                });
-            }
+                }
+                deadbeef->plt_add_files_end (plt, 0);
+                deadbeef->plt_unref (plt);
+                deadbeef->pl_save_current();
+                deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
+            });
+        }
+        else {
+            deadbeef->plt_unref (plt);
         }
     }
 }
@@ -336,11 +445,13 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
                 deadbeef->pl_item_unref (tail);
             }
             deadbeef->plt_add_files_end (plt, 0);
-            if (plt) {
-                deadbeef->plt_unref (plt);
-            }
+            deadbeef->plt_unref (plt);
+            deadbeef->pl_save_current ();
         });
     }
+    else {
+        deadbeef->plt_unref (plt);
+    }
 }
 
 - (IBAction)addLocationCancelAction:(id)sender {
@@ -395,7 +506,7 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
     [_customSortEntry setStringValue:[NSString stringWithUTF8String:deadbeef->conf_get_str_fast ("cocoaui.custom_sort_tf", "")]];
     deadbeef->pl_unlock ();
     [_customSortDescending setState:deadbeef->conf_get_int ("cocoaui.sort_desc", 0) ? NSOnState : NSOffState];
-    [NSApp beginSheet:_customSortPanel modalForWindow:[self mainWindow] modalDelegate:self didEndSelector:@selector(didEndCustomSort:returnCode:contextInfo:) contextInfo:nil];
+    [NSApp beginSheet:_customSortPanel modalForWindow:[_mainWindow window] modalDelegate:self didEndSelector:@selector(didEndCustomSort:returnCode:contextInfo:) contextInfo:nil];
 
 }
 
@@ -506,9 +617,9 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
 }
 
 - (IBAction)stopAfterCurrentAlbumAction:(id)sender {
-    int state = deadbeef->conf_get_int ("playlist.stop_after_current_album", 0);
+    int state = deadbeef->conf_get_int ("playlist.stop_after_album", 0);
     state = 1 - state;
-    deadbeef->conf_set_int ("playlist.stop_after_current_album", state);
+    deadbeef->conf_set_int ("playlist.stop_after_album", state);
     deadbeef->sendmessage (DB_EV_CONFIGCHANGED, 0, 0, 0);
 }
 
@@ -572,6 +683,7 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
     else if (_id == DB_EV_VOLUMECHANGED) {
         [g_appDelegate performSelectorOnMainThread:@selector(volumeChanged) withObject:nil waitUntilDone:NO];
     }
+
     return 0;
 }
 
@@ -627,6 +739,9 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
     if (ctx.plt) {
         deadbeef->plt_unref (ctx.plt);
     }
+    if (it) {
+        deadbeef->pl_item_unref (it);
+    }
     _dockMenuNPTitle = [[NSMenuItem alloc] initWithTitle:[NSString stringWithUTF8String:title] action:nil keyEquivalent:@""];
     [_dockMenuNPTitle setEnabled:NO];
     _dockMenuNPArtistAlbum = [[NSMenuItem alloc] initWithTitle:[NSString stringWithUTF8String:artistAlbum] action:nil keyEquivalent:@""];
@@ -663,16 +778,14 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
         dispatch_queue_t aQueue = dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0);
         dispatch_async(aQueue, ^{
             ddb_playlist_t *plt = deadbeef->plt_get_curr ();
-            if (plt) {
-                if (!deadbeef->plt_add_files_begin (plt, 0)) {
+            if (!deadbeef->plt_add_files_begin (plt, 0)) {
                     deadbeef->plt_clear (plt);
                     int abort = 0;
                     deadbeef->plt_load2 (0, plt, NULL, [fname UTF8String], &abort, NULL, NULL);
                     deadbeef->plt_save_config (plt);
                     deadbeef->plt_add_files_end (plt, 0);
-                }
-                deadbeef->plt_unref (plt);
             }
+            deadbeef->plt_unref (plt);
             deadbeef->sendmessage (DB_EV_PLAYLISTCHANGED, 0, DDB_PLAYLIST_CHANGE_CONTENT, 0);
         });
     }
@@ -731,4 +844,14 @@ static int file_added (ddb_fileadd_data_t *data, void *user_data) {
     return _dockMenu;
 }
 
+- (IBAction)showHelp:(id)sender {
+    if (!_helpWindow) {
+        _helpWindow = [[HelpWindowController alloc] initWithWindowNibName:@"HelpViewer"];
+    }
+
+    if (![[_helpWindow window] isVisible]) {
+        [_helpWindow showWindow:self];
+    }
+}
+
 @end
