@@ -494,6 +494,10 @@ m3uplug_save_m3u (const char *fname, DB_playItem_t *first, DB_playItem_t *last)
     if (!fp) {
         return -1;
     }
+
+    char *tf = deadbeef->tf_compile ("[%artist% - ]%title%");
+    char s[1000];
+
     DB_playItem_t *it = first;
     deadbeef->pl_item_ref (it);
     fprintf (fp, "#EXTM3U\n");
@@ -501,23 +505,21 @@ m3uplug_save_m3u (const char *fname, DB_playItem_t *first, DB_playItem_t *last)
         // skip subtracks, pls and m3u formats don't support that
         uint32_t flags = deadbeef->pl_get_item_flags (it);
         if (flags & DDB_IS_SUBTRACK) {
-            DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
-            deadbeef->pl_item_unref (it);
-            it = next;
-            continue;
+            if (deadbeef->pl_find_meta_int (it, ":TRACKNUM", 0)) {
+                DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
+                deadbeef->pl_item_unref (it);
+                it = next;
+                continue;
+            }
         }
         int dur = (int)floor(deadbeef->pl_get_item_duration (it));
-        char s[1000];
-        int has_artist = deadbeef->pl_meta_exists (it, "artist");
-        int has_title = deadbeef->pl_meta_exists (it, "title");
-        if (has_artist && has_title) {
-            deadbeef->pl_format_title (it, -1, s, sizeof (s), -1, "%a - %t");
-            fprintf (fp, "#EXTINF:%d,%s\n", dur, s);
-        }
-        else if (has_title) {
-            deadbeef->pl_format_title (it, -1, s, sizeof (s), -1, "%t");
-            fprintf (fp, "#EXTINF:%d,%s\n", dur, s);
-        }
+        ddb_tf_context_t ctx = {
+            ._size = sizeof (ddb_tf_context_t),
+            .it = it,
+        };
+        deadbeef->tf_eval (&ctx, tf, s, sizeof (s));
+        fprintf (fp, "#EXTINF:%d,%s\n", dur, s);
+
         deadbeef->pl_lock ();
         {
             const char *fname = deadbeef->pl_find_meta (it, ":URI");
@@ -533,6 +535,8 @@ m3uplug_save_m3u (const char *fname, DB_playItem_t *first, DB_playItem_t *last)
         it = next;
     }
     fclose (fp);
+
+    deadbeef->tf_free (tf);
     return 0;
 }
 
@@ -550,10 +554,12 @@ m3uplug_save_pls (const char *fname, DB_playItem_t *first, DB_playItem_t *last)
         // skip subtracks, pls and m3u formats don't support that
         uint32_t flags = deadbeef->pl_get_item_flags (it);
         if (flags & DDB_IS_SUBTRACK) {
-            DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
-            deadbeef->pl_item_unref (it);
-            it = next;
-            continue;
+            if (deadbeef->pl_find_meta_int (it, ":TRACKNUM", 0)) {
+                DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
+                deadbeef->pl_item_unref (it);
+                it = next;
+                continue;
+            }
         }
         n++;
         if (it == last) {
@@ -574,10 +580,12 @@ m3uplug_save_pls (const char *fname, DB_playItem_t *first, DB_playItem_t *last)
         // skip subtracks, pls and m3u formats don't support that
         uint32_t flags = deadbeef->pl_get_item_flags (it);
         if (flags & DDB_IS_SUBTRACK) {
-            DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
-            deadbeef->pl_item_unref (it);
-            it = next;
-            continue;
+            if (deadbeef->pl_find_meta_int (it, ":TRACKNUM", 0)) {
+                DB_playItem_t *next = deadbeef->pl_get_next (it, PL_MAIN);
+                deadbeef->pl_item_unref (it);
+                it = next;
+                continue;
+            }
         }
         deadbeef->pl_lock ();
         {
@@ -615,8 +623,7 @@ m3uplug_save (ddb_playlist_t *plt, const char *fname, DB_playItem_t *first, DB_p
 
 static const char * exts[] = { "m3u", "m3u8", "pls", NULL };
 DB_playlist_t plugin = {
-    .plugin.api_vmajor = 1,
-    .plugin.api_vminor = 0,
+    DDB_PLUGIN_SET_API_VERSION
     .plugin.version_major = 1,
     .plugin.version_minor = 0,
     .plugin.type = DB_PLUGIN_PLAYLIST,
